/*
 * TODO: header
 *
 * Compilation (with gcc): gcc -O3 tiffreadasc.c tiffwrite.c -lm -ltiff -o tiffreadasc
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#define VERSION "1.0"

#define FREE_ALL() {				\
    if(NULL != u) free(u);			\
  }
#define ASSERT_ALLOC(cmd) if(NULL == (cmd)) { printf("Not enough memory\n"); FREE_ALL(); exit(EXIT_FAILURE); }

static void display_usage()
{
  printf("\nRead an image in ascii format (in row-major order)\n\n");
  printf("Usage: tiffreadasc [-f] out width height\n\n");
  printf("   -f        : save output as a tiff-float instead of unsigned char tiff\n");
  printf("   out       : output image\n");
  printf("   witdh     : image width\n");
  printf("   height    : image height\n\n");
}


int main(int argc, char **argv)
{
  char *filename_out = NULL, *widthvalue = NULL, *heightvalue = NULL, vflag, fflag;
  double *u;
  int i,adr,nx,ny;
  vflag = fflag = 0;

  /******************************************/
  /*            arguments parser            */
  /******************************************/

  /* main parser */
  for(i=1;i<argc;i++) {
    if (strcmp(argv[i],"-f") == 0) fflag = 1;
    else if (NULL == filename_out) filename_out = argv[i];
    else if (NULL == widthvalue) widthvalue = argv[i];
    else if (NULL == heightvalue) heightvalue = argv[i];
    else { display_usage(); printf("Error: could not retrieve properly input arguments.\n\n"); return EXIT_FAILURE; }
  }

  /* check consistency */
  if ((NULL == filename_out) || (NULL == widthvalue) || (NULL == heightvalue)) {
    display_usage();
    if (NULL == filename_out) printf("Error: output 'out' is missing\n\n");
    else if (NULL == widthvalue) printf("Error: input 'width' is missing\n\n");
    else printf("Error: input 'height' is missing\n\n");
    return EXIT_FAILURE;
  }

  if(sscanf(widthvalue,"%d",&nx) != 1) { display_usage(); printf("Error: could not retrieve properly input 'width'\n\n"); return EXIT_FAILURE; }
  if(sscanf(heightvalue,"%d",&ny) != 1) { display_usage(); printf("Error: could not retrieve properly input 'height'\n\n"); return EXIT_FAILURE; }

  /* memory allocation */
  ASSERT_ALLOC(u = (double*) malloc (nx*ny * sizeof(double)));

  /*** main loop  ***/
  for(adr=0;adr<nx*ny;adr++) {
    if(scanf("%lf",u+adr) != 1) {
      printf("\nError: some image graylevels are missing or could not be read properly.\n\n");
      return EXIT_FAILURE;
    }
  }

  /* write tiff image */
  if (EXIT_FAILURE == save_tiff(filename_out,u,nx,ny,8,fflag,1,vflag)) {
    printf("An error occured during the image savinf process.\n");
    printf("Free memory and exit the program.\n\n");
    FREE_ALL();
    return EXIT_FAILURE;
  }

  /* free memory */
  FREE_ALL();

}
